@@ -232,9 +232,7 @@ void Cli7zTest::testList()
     QCOMPARE(entry->property("size").toULongLong(), expectedSize);
 
     QFETCH(QString, expectedTimestamp);
-#if QT_VERSION >= QT_VERSION_CHECK(5, 11, 0)
     QEXPECT_FAIL("", "Something changed since Qt 5.11, needs investigation.", Continue);
-#endif
     QCOMPARE(entry->property("timestamp").toString(), expectedTimestamp);
 
     plugin->deleteLater();
