@@ -25,6 +25,8 @@
 #include "archivemodel.h"
 #include "ark_debug.h"
 #include "jobs.h"
+#include "util.h"
+#include "qstringtokenizer.h"
 
 #include <KIO/Global>
 #include <KLocalizedString>
@@ -32,7 +34,6 @@
 #include <QApplication>
 #include <QDBusConnection>
 #include <QMimeData>
-#include <QMimeDatabase>
 #include <QRegularExpression>
 #include <QStyle>
 #include <QUrl>
@@ -41,7 +42,7 @@ using namespace Kerfuffle;
 
 // Used to speed up the loading of large archives.
 static Archive::Entry *s_previousMatch = nullptr;
-Q_GLOBAL_STATIC(QStringList, s_previousPieces)
+Q_GLOBAL_STATIC(QString, s_previousPath)
 
 ArchiveModel::ArchiveModel(const QString &dbusPathName, QObject *parent)
     : QAbstractItemModel(parent)
@@ -361,41 +362,25 @@ void ArchiveModel::initRootEntry()
 
 Archive::Entry *ArchiveModel::parentFor(const Archive::Entry *entry, InsertBehaviour behaviour)
 {
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    QStringList pieces = entry->fullPath().split(QLatin1Char('/'), QString::SkipEmptyParts);
-#else
-    QStringList pieces = entry->fullPath().split(QLatin1Char('/'), Qt::SkipEmptyParts);
-#endif
-    if (pieces.isEmpty()) {
-        return nullptr;
+    QString fullPath = entry->fullPath();
+
+    if (fullPath.endsWith(QLatin1Char('/'))) {
+        fullPath = fullPath.chopped(1);
     }
-    pieces.removeLast();
 
     // Used to speed up loading of large archives.
-    if (s_previousMatch) {
-        // The number of path elements must be the same for the shortcut
-        // to work.
-        if (s_previousPieces->count() == pieces.count()) {
-            bool equal = true;
+    const int index = fullPath.lastIndexOf(QLatin1Char('/'));
+    const QString folderPath = index != -1 ? fullPath.left(index) : QString();
 
-            // Check if all pieces match.
-            for (int i = 0; i < s_previousPieces->count(); ++i) {
-                if (s_previousPieces->at(i) != pieces.at(i)) {
-                    equal = false;
-                    break;
-                }
-            }
-
-            // If match return it.
-            if (equal) {
-                return s_previousMatch;
-            }
-        }
+    if (s_previousMatch && *s_previousPath == folderPath) {
+        return s_previousMatch;
     }
 
     Archive::Entry *parent = m_rootEntry.data();
 
-    for (const QString &piece : qAsConst(pieces)) {
+    const auto pieces = QStringTokenizer{folderPath, QLatin1Char('/'), Qt::SkipEmptyParts};
+
+    for (const auto piece : pieces) {
         Archive::Entry *entry = parent->find(piece);
         if (!entry) {
             // Directory entry will be traversed later (that happens for some archive formats, 7z for instance).
@@ -420,7 +405,7 @@ Archive::Entry *ArchiveModel::parentFor(const Archive::Entry *entry, InsertBehav
     }
 
     s_previousMatch = parent;
-    *s_previousPieces = pieces;
+    *s_previousPath = folderPath;
 
     return parent;
 }
@@ -443,11 +428,7 @@ void ArchiveModel::slotEntryRemoved(const QString & path)
         return;
     }
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    Archive::Entry *entry = m_rootEntry->findByPath(entryFileName.split(QLatin1Char('/'), QString::SkipEmptyParts));
-#else
     Archive::Entry *entry = m_rootEntry->findByPath(entryFileName.split(QLatin1Char('/'), Qt::SkipEmptyParts));
-#endif
     if (entry) {
         Archive::Entry *parent = entry->getParent();
         QModelIndex index = indexForEntry(entry);
@@ -544,12 +525,7 @@ void ArchiveModel::newEntry(Archive::Entry *receivedEntry, InsertBehaviour behav
     Archive::Entry *parent = parentFor(receivedEntry, behaviour);
 
     // Create an Archive::Entry.
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-    const QStringList path = entryFileName.split(QLatin1Char('/'), QString::SkipEmptyParts);
-#else
-    const QStringList path = entryFileName.split(QLatin1Char('/'), Qt::SkipEmptyParts);
-#endif
-    Archive::Entry *entry = parent->find(path.last());
+    Archive::Entry *entry = parent->find(Kerfuffle::Util::lastPathSegment(entryFileName));
     if (entry) {
         entry->copyMetaData(receivedEntry);
         entry->setProperty("fullPath", entryFileName);
@@ -599,7 +575,7 @@ void ArchiveModel::reset()
 {
     m_archive.reset(nullptr);
     s_previousMatch = nullptr;
-    s_previousPieces->clear();
+    s_previousPath->clear();
     initRootEntry();
 
     // TODO: make sure if it's ok to not have calls to beginRemoveColumns here
@@ -749,11 +725,7 @@ bool ArchiveModel::conflictingEntries(QList<const Archive::Entry*> &conflictingE
     // We can't accept destination as an argument, because it can be a new entry path for renaming.
     const Archive::Entry *destination;
     {
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        QStringList destinationParts = entries.first().split(QLatin1Char('/'), QString::SkipEmptyParts);
-#else
         QStringList destinationParts = entries.first().split(QLatin1Char('/'), Qt::SkipEmptyParts);
-#endif
         destinationParts.removeLast();
         if (destinationParts.count() > 0) {
             destination = m_rootEntry->findByPath(destinationParts);
@@ -776,11 +748,7 @@ bool ArchiveModel::conflictingEntries(QList<const Archive::Entry*> &conflictingE
         }
 
         bool isDir = entry.right(1) == QLatin1String("/");
-#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
-        const Archive::Entry *archiveEntry = lastDirEntry->find(entry.split(QLatin1Char('/'), QString::SkipEmptyParts).last());
-#else
         const Archive::Entry *archiveEntry = lastDirEntry->find(entry.split(QLatin1Char('/'), Qt::SkipEmptyParts).last());
-#endif
 
         if (archiveEntry != nullptr) {
             if (archiveEntry->isDir() != isDir || !allowMerging) {
